#!/usr/bin/php4 -q 
<?php

  /*
   *  $Id: snmpwalk.php,v 1.59 2007-10-29 10:33:16 marlow Exp $
   *
   *  SNMP autodiscovery daemon
   *
   */

  error_reporting(E_ERROR | E_PARSE);

  require_once("DB.php");

  // private networks off grid
  //
  function private_network($ip)
  {
    $rfc1918 = FALSE;
    
    // 192.168/16 range
    if(ereg("^192\.168\.", $ip))    $rfc1918 = TRUE;
    
    // 172.16/12 range
    if(ereg("^172\.1[6-9]\.", $ip)) $rfc1918 = TRUE;
    if(ereg("^172\.2[0-9]\.", $ip)) $rfc1918 = TRUE;
    if(ereg("^172\.3[01]\.", $ip))  $rfc1918 = TRUE;

    // commented out, because we use this range in the Wan

    // 10/8 range
    if(ereg("^10\.", $ip))     $rfc1918 = TRUE;
    
    return $rfc1918;
  }

  // convert IP-address to numeric
  //
  function ipnum($ip)
  {
    // ip-address = w.x.y.z

    // w
    $pos   = strpos($ip, ".");
    $ipnum = (16777216*(intval(substr($ip, 0, $pos))));
    // x
    $opos  = $pos;
    $pos   = strpos($ip, ".", $opos+1);
    $ipnum = ($ipnum + (65536*(intval(substr($ip, $opos+1, $pos-$opos-1)))));
    // y
    $opos  = $pos;
    $pos   = strpos($ip, ".", $opos+1);
    $ipnum = ($ipnum + (256*(intval(substr($ip, $opos+1, $pos-$opos-1)))));
    // z
    $opos  = $pos;
    $ipnum = ($ipnum + intval(substr($ip, $opos+1)));

    return $ipnum;
  } 

  // defaults
  $host_community = "public";
  $sql_host       = "localhost";
  $sql_user       = "snmpwalk";
  $sql_pass       = "snmpwalk";
  $sql_db         = "wifimap";
  $sql_prefix     = "";
  $debug          = FALSE;
  $help           = FALSE;
  $noscan         = TRUE;
  $nowalk         = FALSE;
  $silent         = FALSE;
  $discovery      = FALSE;

  // header
  $header = $GLOBALS[argv][0]." - written 2005-2007 by Martin List-Petersen - marlow (at) thewan (dot) net\n\n";

  // check for parameters on commandline
  //
  for($x=1; $x<count($GLOBALS[argv]); $x++)
    switch($GLOBALS[argv][$x])
    {
      case "--debug":
                      // print debug information, if silent wsn't enabled at the same time
	              //
	              if($silent == FALSE) $debug = TRUE;
	              break;
	              
      case "--discovery":
                      // discovery mode
	              //
	              $discovery = TRUE;
	              break;
	              
      case "-d":
                      // discovery mode, short option
	              //
	              $discovery = TRUE;
	              break;
	              
      case "--help":
                      // either help was asked or no arguments specified.
                      // print syntax
                      //
                      $help = TRUE;
                      break;

      case "--nodb":
	              // disable use of database
	              //
	              $sql_host = "";
	              break;
	              
      case "--nowalk":
	              // disable use of database
	              //
	              $nowalk = TRUE;
	              break;
	              
      case "--scan":
                      // don't use pingscan for arp populating
                      $noscan = FALSE;
                      break;

      case "--silent":
                      // silent mode, don't print anything, not even debug
                      //
                      $silent = TRUE;
                      $debug  = FALSE;
                      break;

      case "-u":
                      $sql_user = $GLOBALS[argv][$x+1];
	              $x++;
	              break;
	              
      case "-p":
	              $sql_pass = $GLOBALS[argv][$x+1];
	              $x++;
	              break;
	              
      case "-h":
	              $sql_host = $GLOBALS[argv][$x+1];
	              $x++;
	              break;

      case "--db":
	              $sql_db = $GLOBALS[argv][$x+1];
	              $x++;
	              break;    

      case "-c":
	              // snmp community, currently only one in a run supported
	              //
	              $host_community = $GLOBALS[argv][$x+1];
	              $x++;
	              break;    

      default:
	              // host to query first
	              //
	              $host_ip = $GLOBALS[argv][$x];
    }

  // ip needs to be specified for discovery mode
  //
  if($discovery == TRUE AND $host_ip == "") $help = TRUE;

  //
  // help output
  //
  if($help == TRUE)
  {
    print $header;
    print "SYNTAX: ".$GLOBALS[argv][0]." [options] [ip]\n\n";
    print "OPTIONS:\n";
    print "            --debug     enabled debugging output\n";
    print "            --discovery discovery mode, IP needs to be specified. short: -d\n";
    print "            --help      this help screen\n";
    print "            --nodb      don't store information in the database\n";
    print "            --scan      use pingscan to populate routers arp table (EVIL !!)\n";
    print "            --silent    silent mode, no output whatsoever\n";
    print "            -u          user for database (default: snmpwalk)\n";
    print "            -p          password for database (default: snmpwalk)\n";
    print "            -h          host for database (default: localhost)\n";
    print "            --db        database to use (default: wifimap)\n";
    print "            -c          community to use (default: public)\n";
    print "\n";
    die;
  }

  if($silent == FALSE) print $header;

	if($sql_host != "")
	{
	  $db = DB::connect("mysql://".$sql_user.":".$sql_pass."@tcp+".$sql_host."/".$sql_db);
	  if (DB::isError($db)) {
	    if($silent == FALSE) print "\nDatabase not available !!\n";
	    $sql = FALSE;
	  } else
	    $sql = TRUE;
  } else {
    if($silent == FALSE) print "\nNot using database !!\n";
    $sql = FALSE;
  }

  function mac_pad($mac)
  {
    $mac_bits = explode(":", substr($mac, 8));
    
    $mac = "";
    for($x=0; $x < count($mac_bits); $x++)
    {
      if(strlen($mac_bits[$x]) == "") $mac .= "0";
      if(strlen($mac_bits[$x]) == 1) $mac .= "0";
      $mac .= $mac_bits[$x];
      if($x < count($mac_bits)-1) $mac .=":";
    }
    
    return $mac;
  }

  function walk_node($host, $host_community)
  {
    GLOBAL $db, $debug, $silent, $noscan, $sql, $sql_prefix;
    
    if($silent == FALSE) print "\n\n".$host." (".ip2long($host).")\n\n";
  
    // query host for active ethernet interfaces
    //
    $interfaces = snmpwalk($host, $host_community, ".1.3.6.1.2.1.2.2.1.3");
    if($debug)
    {
      print_r($interfaces);
      print "\n";
    }

    if(count($interfaces) > 1)
    {
      // populate arp table via pingscan
      //
      $route_info = snmprealwalk($host, $host_community, ".1.3.6.1.2.1.4.20.1.3");
      $route_mib  = array_keys($route_info);
      $route_val  = array_values($route_info);
      
      // only execute pingscan on anything less than a-class network, not single hosts and not defaultroute and localhost
      //
      if($noscan != TRUE)
        for($x=0; $x < count($route_mib); $x++)
          if((substr($route_mib[$x], 23) != "0.0.0.0") AND (substr($route_mib[$x], 23) != "127.0.0.1") AND ($route_val[$x] != "IpAddress: 255.0.0.0") AND ($route_val[$x] != "IpAddress: 255.255.255.255"))
          {
            $cmd = "nmap -sP `ipcalc ".substr($route_mib[$x], 23)."/".substr($route_val[$x], 11)." | grep Network | cut -f 4 -d ' '`";
            if($debug == FALSE) $cmd .= " >/dev/null 2>&1";
            if($debug == TRUE) print $cmd."\n";
            $fp = popen($cmd, "r");
            fpassthru($fp);
          }

      // query active interfaces
      //
      $int_status  = snmpwalk($host, $host_community, ".1.3.6.1.2.1.2.2.1.7");
      $int_index   = snmpwalk($host, $host_community, ".1.3.6.1.2.1.2.2.1.1");
      $int_mac     = snmpwalk($host, $host_community, ".1.3.6.1.2.1.2.2.1.6");
      // get router uptime
      //
      $host_uptime = snmpwalk($host, $host_community, ".1.3.6.1.2.1.1.3");
      $int_uptime  = substr($host_uptime[0], (strpos($host_uptime[0], "(")+1), (strpos($host_uptime[0], ")")-strpos($host_uptime[0], "(")-1));
      if($silent == FALSE) print "Uptime: ".$int_uptime." - ".$host_uptime[0]."\n";
      //
      $int_ether  = array();
      $int_stat   = array();
      for($x=0; $x < count($interfaces); $x++)
        if(($interfaces[$x] == "INTEGER: ethernetCsmacd(6)") OR ($interfaces[$x] == "INTEGER: ieee80211(71)") OR ($interfaces[$x] == "INTEGER: bridge(209)") OR ($interfaces[$x] == "INTEGER: tunnel(131)"))
          if (mac_pad($int_mac[$x]) != "0")
          {
            $int_ether[] = substr($int_index[$x], 9);
            $int_stat[]  = substr($int_status[$x], 9, strpos($int_status[$x], "(")-9);
          }

      if($debug == TRUE) 
      {
        print "\n";
        print_r($int_mac);
        print "\n";
      }

      // ip adresses
      $ip_info    = snmprealwalk($host, $host_community, ".1.3.6.1.2.1.4.20.1.2");
      $ip_mib     = array_keys($ip_info);
      $ip_val     = array_values($ip_info);
      // netmasks
      $ip_info    = snmprealwalk($host, $host_community, ".1.3.6.1.2.1.4.20.1.3");
      $ip_nm_mib  = array_keys($ip_info);
      $ip_nm_val  = array_values($ip_info);
  
      // query host for interface data
      //
      $host_info = array();
      for($x=0; $x < count($int_ether); $x++)
      {
        if($debug) print $x." - ";
        $host_info[$x] = array();
        $host_info[$x]["int"] = substr(snmpget($host, $host_community, ".1.3.6.1.2.1.2.2.1.2.".($int_ether[$x])), 8);
        if($debug) print  ".1.3.6.1.2.1.2.2.1.2.".($int_ether[$x])."\n";
        $host_info[$x]["mac"] = mac_pad(snmpget($host, $host_community, ".1.3.6.1.2.1.2.2.1.6.".$int_ether[$x]));
        $host_info[$x]["status"] = $int_stat[$x];
        if($host_info[$x]["mac"] == "0")
        {
          if($silent == FALSE)
            print "Broken interface !!\n";
          break;
        }
        $host_info[$x]["ip"] = array();
        $host_info[$x]["netmask"] = array();
        for($y=0; $y < count($ip_val); $y++)
          if($ip_val[$y] == "INTEGER: ".$int_ether[$x])
          {
            $ip_str                     = snmpget($host, $host_community, ".1.3.6.1.2.1.4.20.1.1.".substr($ip_mib[$y], strpos($ip_mib[$y], ".")+1));
            if(!private_network(substr($ip_str, strpos($ip_str, ":")+2)))
            {
              $host_info[$x]["ip"][]      = substr($ip_str, strpos($ip_str, ":")+2);
              $host_info[$x]["netmask"][] = substr($ip_nm_val[$y], strpos($ip_nm_val[$y], ":")+2);
            }
          }
      }

      if($silent == FALSE) 
      {
        print "\n";
        print_r($host_info);
        print "\n";
      }

      // update database
      //
      if($sql == TRUE)
      {
        // node allready known to database ??
        //
        // make sure that RFC1918 and empty adresses are ignored
        //
        $host_list = "";
        for($x=0; $x < count($host_info); $x++)
          for($y=0; $y < count($host_info[$x]["ip"]); $y++)
            if(($host_info[$x]["ip"][$y] != "") AND private_network($host_info[$x]["ip"][$y]) != TRUE)
              $host_list .= "'".ip2long($host_info[$x]["ip"][$y])."',";

        if(substr($host_list, strlen($host_list)-1) == ",") $host_list = substr($host_list, 0, strlen($host_list)-1);
        
        $query  = "SELECT id,ip,nip FROM ".$sql_prefix."snmp_mac WHERE ip IN (".$host_list.")";
        if(strlen($host_list) > 0)
        {
          $result = $db->query($query);
          if($debug == TRUE) print $query."\n";
          if(DB::isError($result)) die("Died: ".$query."\n");

          if($result->numRows() >= 1) 
          {
            // it is, get the id
            //
            while($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
            {
              $host_id  = $row["node_id"];
              $host_nip = $row["nip"];

              if(($host_id == 0) AND ($row["ip"] != -1) AND (private_network($row["ip"]) != TRUE))
              {
                $query    = "SELECT id FROM ".$sql_prefix."snmp_node WHERE ip='".$row["ip"]."'";
                $qhostid  = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($qhostid)) die("Died: ".$query."\n");
                if($qhostid->numRows() < 1)
                {
                  // ip not known to node database yet
                  //
                  $query   = "INSERT INTO ".$sql_prefix."snmp_node (ip, date_seen) VALUES('".$row["ip"]."','".time()."')";
                  $qhostid  = $db->query($query);
                  if($debug == TRUE) print $query."\n";
                  if(DB::isError($qhostid)) die("Died: ".$query."\n");

                  $query  = "SELECT id FROM ".$sql_prefix."snmp_node WHERE ip='".$row["ip"]."'";
                  $qhostid  = $db->query($query);
                  if($debug == TRUE) print $query."\n";
                  if(DB::isError($qhostid)) die("Died: ".$query."\n");
                }
                $noderow = $qhostid->fetchRow(DB_FETCHMODE_ASSOC);
                $host_id = $noderow["id"];

                $query    = "UPDATE ".$sql_prefix."snmp_mac SET date_seen='".time()."',node_id='".$host_id."' WHERE ip IN (".$host_list.")";
                $qhostid  = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($qhostid)) die("Died: ".$query."\n");
              } else {
                $query    = "UPDATE ".$sql_prefix."snmp_node SET date_seen='".time()."' WHERE id='".$host_id."'";
                $qhostid  = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($qhostid)) die("Died: ".$query."\n");
              }
          
              if($host_nip != 0)
              {
                // check current status
                //
                $query    = "SELECT node_status FROM ".$sql_prefix."node_ips WHERE id='".$host_nip."'";
                $qhostnip = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($qhostnip)) die("Died: ".$query."\n");
                $row      = $qhostnip->fetchRow(DB_FETCHMODE_ASSOC);
                if($row["node_status"] == "down" OR $row["node_status"] == "unknown")
                  $query  = "UPDATE ".$sql_prefix."node_ips SET node_status='up',node_uptime='".$int_uptime."',date_status='".time()."',date_seen='".time()."' WHERE id='".$host_nip."'";
                else
                  $query  = "UPDATE ".$sql_prefix."node_ips SET node_uptime='".$int_uptime."',date_seen='".time()."' WHERE id='".$host_nip."'";
                $qhostnip = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($qhostnip)) die("Died: ".$query."\n");
              }
            }
          } else {
            // it's not, create it with the first best ip and get the id
            //
            for($x=0; $x < count($host_info); $x++)
              for($y=0; $y < count($host_info[$x]["ip"]); $y++)
                if((private_network($host_info[$x]["ip"][$y]) == FALSE) AND ($host_info[$x]["ip"][$y] != ""))
                {
                  $query   = "INSERT INTO ".$sql_prefix."snmp_node (ip, date_seen) VALUES('".ip2long($host_info[$x]["ip"][$y])."','".time()."')";
                  $qhostid  = $db->query($query);
                  if($debug == TRUE) print $query."\n";
                  if(DB::isError($qhostid)) die("Died: ".$query."\n");

                  $query  = "SELECT id FROM ".$sql_prefix."snmp_node WHERE ip='".ip2long($host_info[$x]["ip"][$y])."'";
                  $qhostid  = $db->query($query);
                  if($debug == TRUE) print $query."\n";
                  if(DB::isError($qhostid)) die("Died: ".$query."\n");
                  $row     = $qhostid->fetchRow(DB_FETCHMODE_ASSOC);
                  $host_id = $row["id"];

                  $x       = count($host_info);
                  $y       = count($host_info[$x]["ip"]);
                } 
          }

          if($silent == FALSE) print "Node ID: ".$host_id."\n\n";
          if($host_id == "")
          {
            print "ResultNum: ".$result->numRows()."\n\n";
            die;
          }

          // remove all ip data, as we're refreshing here
          //
          $query  = "DELETE FROM ".$sql_prefix."snmp_mac where node_id='".$host_id."' AND int_status != 'manual'";
          $result = $db->query($query);
          if($debug == TRUE) print $query."\n";
          if(DB::isError($result)) die("Died: ".$query."\n");

          // repopulate node
          //
          for($x=0; $x < count($host_info); $x++)
            for($y=0; $y < count($host_info[$x]["ip"]); $y++)
              if((private_network($host_info[$x]["ip"][$y]) == FALSE) AND ($host_info[$x]["ip"][$y] != ""))
              {
                //
                // network details
                //
                $mac_ip        = ip2long($host_info[$x]["ip"][$y]);
                $mac_netmask   = ip2long($host_info[$x]["netmask"][$y]);
                $mac_network   = ($mac_ip & $mac_netmask);
                $mac_broadcast = $mac_network | (~$mac_netmask);
                
                $network       = long2ip($mac_network);
                $broadcast     = long2ip($mac_broadcast);

                if($debug == TRUE)
                {
                  print "IP       : ".long2ip($mac_ip)."\n";
                  print "Netmask  : ".long2ip($mac_netmask)."\n";
                  print "Network  : ".$network."\n";
                  print "Broadcast: ".$broadcast."\n\n";
                  print "Network  : ".ipnum($network)."\n";
                  print "Broadcast: ".ipnum($broadcast)."\n\n";
                }

                $query = "SELECT id,int_status,date_status FROM ".$sql_prefix."snmp_mac WHERE ip='".$mac_ip."'";
                $result = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($result)) die("Died: ".$query."\n");
                if($result->numRows() < 1)
                  $query = "INSERT INTO ".$sql_prefix."snmp_mac (ip, netmask, network, broadcast, mac, interface, int_status, node_id, date_status, date_seen) VALUES('".$mac_ip."','".$mac_netmask."','".ipnum($network)."','".ipnum($broadcast)."','".$host_info[$x]["mac"]."','".$host_info[$x]["int"]."','".$host_info[$x]["status"]."','".$host_id."','".time()."','".time()."')";
                else {
                  $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
                  if($row["int_status"] != $host_info[$x]["status"])
                    $date_status = time();
                  else
                    $date_status = $row["date_status"];
                  $query = "UPDATE ".$sql_prefix."snmp_mac SET netmask='".$mac_netmask."',network='".ipnum($network)."',broadcast='".ipnum($broadcast)."',mac='".$host_info[$x]["mac"]."',interface='".$host_info[$x]["int"]."',int_status='".$host_info[$x]["status"]."',node_id='".$host_id."',date_status='".$date_status."',date_seen='".time()."' WHERE ip='".$mac_ip."'";
                }
                $result = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($result)) die("Died: ".$query."\n");
              }
          
          // re-establish link data
          //
          $query  = "SELECT N.id,SM.node_id,NI.id AS node_ip,SM.id AS mac_id FROM ".$sql_prefix."nodes AS N INNER JOIN ".$sql_prefix."node_ips AS NI ON N.id=NI.node_id LEFT JOIN ".$sql_prefix."snmp_mac AS SM ON SM.ip=NI.ip WHERE SM.node_id IS NOT NULL AND SM.node_id = '".$host_id."'";
          // GROUP BY SM.node_id";
          $q_nid  = $db->query($query);
          if(DB::isError($q_nid)) die("Died: ".$query);

          while($row = $q_nid->fetchRow(DB_FETCHMODE_ASSOC))
          {
            $query = "UPDATE ".$sql_prefix."snmp_mac SET nid='".$row["id"]."' WHERE node_id='".$row["node_id"]."'";
            $q_row = $db->query($query);
            if(DB::isError($q_row)) die("Died: ".$query);
            $query = "UPDATE ".$sql_prefix."snmp_mac SET nip='".$row["node_ip"]."' WHERE node_id='".$row["node_id"]."'";
            $q_row = $db->query($query);
            if(DB::isError($q_row)) die("Died: ".$query);
          }

          // get linked hosts ips
          //
          $link_query = snmpwalk($host, $host_community, ".1.3.6.1.2.1.4.22.1.3");
          $link_keys  = array_values($link_query);
  
          for($x=0; $x < count($link_keys); $x++)
          {
            $link_host = substr($link_keys[$x], strpos($link_keys[$x], ":")+2);

            $query  = "SELECT id FROM ".$sql_prefix."snmp_mac WHERE ip='".ip2long($link_host)."'";
            $result = $db->query($query);
            if($debug == TRUE) print $query."\n";
            if(DB::isError($result)) die("Died: ".$query."\n");
            if($result->numRows() >= 1)
            {
              $query = "UPDATE ".$sql_prefix."snmp_mac SET date_seen='".time()."' WHERE ip='".ip2long($link_host)."'";
              if($silent == FALSE) print $link_host."\n";
            } else 
              if($link_host != "" AND private_network($link_host) == FALSE)
              {
                $query = "INSERT INTO ".$sql_prefix."snmp_mac (ip, date_seen) VALUES('".ip2long($link_host)."','".time()."')";
                if($silent != TRUE) print "Adding '".$link_host."' to database. 459.\n";
              }
            $result = $db->query($query);
            if($debug == TRUE) print $query."\n";
            if(DB::isError($result)) die("Died: ".$query."\n");
          }
        } else {
          // we didn't get any IP's. SNMP is screwed up
          $query = "SELECT id,node_status FROM ".$sql_prefix."node_ips WHERE ip='".ip2long($host)."'";
          $result = $db->query($query);
          if($debug == TRUE) print $query."\n";
          if(DB::isError($result)) die("Died: ".$query."\n");
          $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
          // set status if necessary            
          if($row["node_status"] == "up" OR $row["node_status"] == "unknown")
            $query = "UPDATE ".$sql_prefix."node_ips SET date_status='".time()."',date_seen='".time()."',node_status='down' WHERE id='".$row["id"]."'";
          else
            $query = "UPDATE ".$sql_prefix."node_ips SET date_seen='".time()."' WHERE id='".$row["id"]."'";
          $result = $db->query($query);
          if($debug == TRUE) print $query."\n";
          if(DB::isError($result)) die("Died: ".$query."\n");
        }
      } else {
        for($x=0; $x < count($link_keys); $x++)
        {
          $link_host = substr($link_keys[$x], strpos($link_keys[$x], ":")+2);
          if($silent == FALSE) print "\n".$link_host;
        }
      }

      if($silent == FALSE) print "\n\n";
    } else {
      if($sql == TRUE)
      {
        $query  = "SELECT id,mac,nip FROM ".$sql_prefix."snmp_mac WHERE ip='".ip2long($host)."'";
        $result = $db->query($query);
        if($debug == TRUE) print $query."\n";
        if(DB::isError($result)) die("Died: ".$query."\n");

        // mark node as client, don't query it anymore
        //
        if(count($interfaces) == 1)
        {
          if($result->numRows() >= 1)
          {
            if($debug == TRUE) print "Found only one interface !!\n";
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if(($row["mac"] == "" OR $row["mac"] == "client") AND ($interfaces[0] != ""))
              $query = "UPDATE ".$sql_prefix."snmp_mac SET date_seen='".time()."',mac='client' WHERE ip='".ip2long($host)."'";
            else {
              // node is down
              //
              if($row["nip"] != 0)
              {
                $query = "SELECT id,node_status FROM ".$sql_prefix."node_ips WHERE id='".$row["nip"]."'";
                $result = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($result)) die("Died: ".$query."\n");
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
                // set status if necessary            
                if($row["node_status"] == "up" OR $row["node_status"] == "unknown")
                  $query = "UPDATE ".$sql_prefix."node_ips SET date_status='".time()."',date_seen='".time()."',node_status='down' WHERE id='".$row["id"]."'";
                else
                  $query = "UPDATE ".$sql_prefix."node_ips SET date_seen='".time()."' WHERE id='".$row["id"]."'";
                $result = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($result)) die("Died: ".$query."\n");
              } else {
                // node is specified in node_ips, but we've never seen it yet.
                // setting it back to 'unknown' status, if it has something else
                //
                $query = "UPDATE ".$sql_prefix."node_ips SET date_status='".time()."',date_seen='".time()."',node_status='unknown' WHERE ip='".ip2long($host)."'";
                $result = $db->query($query);
                if($debug == TRUE) print $query."\n";
                if(DB::isError($result)) die("Died: ".$query."\n");
              }                
            }
          } else {
            $query = "INSERT INTO ".$sql_prefix."snmp_mac (ip, mac, date_seen) VALUES('".ip2long($host)."','client','".time()."')";
            if($silent != TRUE) print "Adding '".$host."' to database. 491.\n";
          }
          $result = $db->query($query);
          if($debug == TRUE) print $query."\n";
          if(DB::isError($result)) die("Died: ".$query."\n");
        } else {
          // we didn't get any data, that means this node down.
          //
          if($result->numRows() >= 1)
          {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            // get status on node, if node_ips.id (nip) has been assigned
            if($row["nip"] != 0)
            {
              $query = "SELECT id,node_status FROM ".$sql_prefix."node_ips WHERE id='".$row["nip"]."'";
              $result = $db->query($query);
              if($debug == TRUE) print $query."\n";
              if(DB::isError($result)) die("Died: ".$query."\n");
              $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
              // set status if necessary            
              if($row["node_status"] == "up" OR $row["node_status"] == "unknown")
                $query = "UPDATE ".$sql_prefix."node_ips SET date_status='".time()."',date_seen='".time()."',node_status='down' WHERE id='".$row["id"]."'";
              else
                $query = "UPDATE ".$sql_prefix."node_ips SET date_seen='".time()."' WHERE id='".$row["id"]."'";
              $result = $db->query($query);
              if($debug == TRUE) print $query."\n";
              if(DB::isError($result)) die("Died: ".$query."\n");
            }
          }
        }
      }
    }
  }

  if($discovery == TRUE)
  {
    // DISCOVERY MODE
    // retrieve data from initial node
    //
    walk_node($host_ip, $host_community);

    if($sql AND !$nowalk)
    {
      // check if we have clients/nodes without mac address, which means we haven't fully traversed the network
      //
      $query = "SELECT SM.ip FROM ".$sql_prefix."snmp_mac AS SM LEFT JOIN ".$sql_prefix."snmp_node AS SN ON SN.ip=SM.ip WHERE SM.mac=''";
      $result = $db->query($query);
      if($debug == TRUE) print $query."\n";
      if(DB::isError($result)) die("Died: ".$query."\n");
      while($result)
      {
        if($result->numRows() == 0) break;
        while($row = $result->fetchRow(DB_FETCHMODE_ASSOC))
          walk_node(long2ip($row["ip"]), $host_community);
        $result = $db->query($query);
        if($debug == TRUE) print $query."\n";
        if(DB::isError($result)) die("Died: ".$query."\n");
      }
    }

    // asssign current nodes in database with snmp_mac database for mapping and geolocation
    //
    $query  = "SELECT N.id,SM.node_id,NI.id AS node_ip,SM.id AS mac_id FROM ".$sql_prefix."nodes AS N INNER JOIN ".$sql_prefix."node_ips AS NI ON N.id=NI.node_id LEFT JOIN ".$sql_prefix."snmp_mac AS SM ON SM.ip=NI.ip WHERE SM.node_id IS NOT NULL AND SM.node_id != 0 ";
    // GROUP BY SM.node_id";
    $q_nid  = $db->query($query);
    if(DB::isError($q_nid)) die("Died: ".$query);

    while($row = $q_nid->fetchRow(DB_FETCHMODE_ASSOC))
    {
          $query = "UPDATE ".$sql_prefix."snmp_mac SET nid='".$row["id"]."' WHERE node_id='".$row["node_id"]."'";
          $q_row = $db->query($query);
          if(DB::isError($q_row)) die("Died: ".$query);
          $query = "UPDATE ".$sql_prefix."snmp_mac SET nip='".$row["node_ip"]."' WHERE node_id='".$row["node_id"]."'";
          $q_row = $db->query($query);
          if(DB::isError($q_row)) die("Died: ".$query);
    }
  } else {
    // STATUS QUERY MODE
    if($host_ip == "")
    {
      // no ip specified, query all in database
      //
      $query = "SELECT ip FROM ".$sql_prefix."node_ips WHERE node_status != 'nocheck' AND node_type != 'nosnmp' ORDER BY ip";
      $q_row = $db->query($query);
      if(DB::isError($q_row)) die("Died: ".$query);
      while($row = $q_row->fetchRow(DB_FETCHMODE_ASSOC))
        walk_node(long2ip($row["ip"]), $host_community);
    } else
      walk_node($host_ip, $host_community);

    // asssign current nodes in database with snmp_mac database for mapping and geolocation
    //
    $query  = "SELECT N.id,SM.node_id,NI.id AS node_ip,SM.id AS mac_id FROM ".$sql_prefix."nodes AS N INNER JOIN ".$sql_prefix."node_ips AS NI ON N.id=NI.node_id LEFT JOIN ".$sql_prefix."snmp_mac AS SM ON SM.ip=NI.ip WHERE SM.node_id IS NOT NULL AND SM.node_id != 0 ";
    // GROUP BY SM.node_id";
    $q_nid  = $db->query($query);
    if(DB::isError($q_nid)) die("Died: ".$query);

    while($row = $q_nid->fetchRow(DB_FETCHMODE_ASSOC))
    {
          $query = "UPDATE ".$sql_prefix."snmp_mac SET nid='".$row["id"]."' WHERE node_id='".$row["node_id"]."'";
          $q_row = $db->query($query);
          if(DB::isError($q_row)) die("Died: ".$query);
          $query = "UPDATE ".$sql_prefix."snmp_mac SET nip='".$row["node_ip"]."' WHERE node_id='".$row["node_id"]."'";
          $q_row = $db->query($query);
          if(DB::isError($q_row)) die("Died: ".$query);
    }
  }

?>